﻿Imports FarPoint.Win.Spread
Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.Drawing.Imaging
Imports System.Reflection
Imports System.Text
Imports System.Windows.Forms
Public Class ImageSpread
    Inherits FpSpread
    ''' <summary>
    ''' Spread have an optimzed performance code, so if there is no background image, it will not invalidate the whole spread bound when scrolling
    ''' only new rows/column area appear after scroll will be repaint.
    ''' This temp image will help it work smooth in case Layout is not Tile
    ''' </summary>
    Private tmpBackgroundImage As Image = New Bitmap(1, 1, PixelFormat.Format16bppArgb1555)

    Public Overrides Property BackgroundImage() As Image
        Get
            If MyBase.BackgroundImage Is Nothing Then
                Return tmpBackgroundImage
            Else
                Return MyBase.BackgroundImage
            End If
        End Get
        Set(value As Image)
            MyBase.BackgroundImage = value
        End Set
    End Property

    Protected Overrides Sub Dispose(disposing As Boolean)
        tmpBackgroundImage.Dispose()
        MyBase.Dispose(disposing)
    End Sub

    Protected Overrides Sub OnPaintBackground(e As PaintEventArgs)
        MyBase.OnPaintBackground(e)
        'need to paint for header area and background in case sheetview might contain transparent image/color and need to make it effects.
        Dim sv As ImageSheetView = TryCast(Me.ActiveSheet, ImageSheetView)

        If sv IsNot Nothing Then
            Dim backColor As Color = sv.BackColor
            Dim image As Image = sv.BackgroundImage
            Dim layout As ImageLayout = sv.BackgroundImageLayout
            If backColor = Color.Empty AndAlso image Is Nothing Then
                Return
            End If
            Dim state As GraphicsState = e.Graphics.Save()

            Dim br As Brush = If(backColor <> Color.Empty AndAlso layout = ImageLayout.Tile, New SolidBrush(backColor), Nothing)
            Dim drawBackgroundImage As MethodInfo = GetType(ControlPaint).GetMethod("DrawBackgroundImage", BindingFlags.NonPublic Or BindingFlags.[Static], Nothing, New Type() {GetType(Graphics), GetType(Image), GetType(Color), GetType(ImageLayout), GetType(Rectangle), GetType(Rectangle), _
             GetType(Point), GetType(RightToLeft)}, Nothing)

            Dim rViewportCnt As Integer = Me.GetRowViewportCount(), cViewportCnt As Integer = Me.GetColumnViewportCount()
            For rowViewport As Integer = 0 To rViewportCnt - 1
                For colViewport As Integer = 0 To cViewportCnt - 1
                    Dim topRow As Integer, leftCol As Integer
                    If (InlineAssignHelper(topRow, Me.GetViewportTopRow(rowViewport))) < sv.RowCount - 1 AndAlso (InlineAssignHelper(leftCol, Me.GetViewportLeftColumn(colViewport))) < sv.ColumnCount - 1 Then
                        ' Implemented it for total no. of rows and columns rather than a cell range
                        Dim n As Integer
                        Dim vpRect As Rectangle = Me.GetViewportRectangle(rowViewport, colViewport)
                        Dim needOffsetPoint As Boolean = image IsNot Nothing AndAlso layout = ImageLayout.Tile
                        Dim scrollOffset As Point = vpRect.Location
                        If needOffsetPoint AndAlso topRow > 0 Then
                            For n = 0 To topRow - 1
                                scrollOffset.Y -= CInt(sv.GetRowHeight(n) * Me.ZoomFactor)
                            Next
                        End If
                        If needOffsetPoint AndAlso leftCol > 0 Then
                            For n = 0 To leftCol - 1
                                scrollOffset.X -= CInt(sv.GetRowHeight(n) * Me.ZoomFactor)
                            Next
                        End If
                        If br IsNot Nothing Then
                            e.Graphics.FillRectangle(br, vpRect)
                        End If
                        drawBackgroundImage.Invoke(Nothing, New Object() {e.Graphics, image, backColor, layout, vpRect, vpRect, _
                         scrollOffset, Me.RightToLeft})
                        e.Graphics.Restore(state)
                    End If
                Next
            Next
            If br IsNot Nothing Then
                br.Dispose()
            End If
            e.Graphics.Restore(state)
        End If
    End Sub
    Private Shared Function InlineAssignHelper(Of T)(ByRef target As T, value As T) As T
        target = value
        Return value
    End Function
End Class
Class ImageSheetView
    Inherits SheetView
    Public Sub New()
        MyBase.New()
        Me.DefaultStyle.BackColor = Color.Transparent
        Me.BackColor = Color.Empty
        Me.BackgroundImage = Nothing
        Me.BackgroundImageLayout = ImageLayout.None
    End Sub
    Public Sub New(backColor As Color, background As Image, layout As ImageLayout)
        Me.New()
        Me.BackColor = backColor
        Me.BackgroundImage = background
        Me.BackgroundImageLayout = layout
    End Sub

    ''' <summary>
    ''' Gets or sets custom back color
    ''' </summary>
    Public Property BackColor() As Color
        Get
            Return m_BackColor
        End Get
        Set(value As Color)
            m_BackColor = Value
        End Set
    End Property
    Private m_BackColor As Color

    ''' <summary>
    ''' Gets or sets custom background image
    ''' </summary>
    Public Property BackgroundImage() As Image
        Get
            Return m_BackgroundImage
        End Get
        Set(value As Image)
            m_BackgroundImage = Value
        End Set
    End Property
    Private m_BackgroundImage As Image
    ''' <summary>
    ''' Gets or sets custom background image layout
    ''' </summary>
    Public Property BackgroundImageLayout() As ImageLayout
        Get
            Return m_BackgroundImageLayout
        End Get
        Set(value As ImageLayout)
            m_BackgroundImageLayout = Value
        End Set
    End Property
    Private m_BackgroundImageLayout As ImageLayout
End Class
